from setuptools import Extension
from setuptools import setup


# These extensions are expected to work anywhere with a:
#
# * Standards-compliant C++ 11 compiler (GCC >= 4.8 or Clang >= 700)
# * OpenFst 1.5.1 (http://openfst.org) built with the "far", "pdt", and "mpdt"
#   extensions.
# * re2 (http:://github.com/google/ee2)
# * Python 2.7 or better with "development" headers


COMPILE_ARGS = ["-std=c++11", "-Wno-unused-function"]

pywrapfst = Extension(name="pywrapfst", language="c++",
                      extra_compile_args=COMPILE_ARGS,
                      libraries=["fstfarscript", "fstfar", "fstscript",
                                 "fst", "m", "dl"],
                      sources=["src/pywrapfst.cc"])

pynini = Extension(name="pynini", language="c++",
                   extra_compile_args=COMPILE_ARGS,
                   libraries=["re2",
                              "fstfarscript",
                              "fstpdtscript",
                              "fstmpdtscript",
                              "fstscript",
                              "fstfar",
                              "fst",
                              "m",
                              "dl"],
                    sources=["src/crossproductscript.cc",
                             "src/join.cc",
                             "src/merge.cc",
                             "src/mergescript.cc",
                             "src/optimizescript.cc",
                             "src/pathsscript.cc",
                             "src/pynini.cc",
                             "src/pynini_cdrewrite.cc",
                             "src/pynini_replace.cc",
                             "src/pynini_stringify.cc",
                             "src/repeatscript.cc",
                             "src/split.cc",
                             "src/stringcompile.cc",
                             "src/stringcompilescript.cc"])

setup(
    name="pynini", version="0.1",
    description="Finite-state grammar compilation library",
    author="Kyle Gorman", author_email="kbg@google.com",
    url="http://pynini.opengrm.org/",
    keywords=["natural language processing", "speech recognition",
              "machine learning"],
    classifiers=["Programming Language :: Python :: 2.7",
                 "Development Status :: 4 - Beta",
                 "Environment :: Other Environment",
                 "Environment :: Console",
                 "Intended Audience :: Developers",
                 "License :: OSI Approved :: Apache Software License",
                 "Operating System :: OS Independent",
                 "Topic :: Software Development :: Libraries :: Python Modules",
                 "Topic :: Text Processing :: Linguistic",
                 "Topic :: Scientific/Engineering :: Artificial Intelligence",
                 "Topic :: Scientific/Engineering :: Mathematics"],
    ext_modules=[pywrapfst, pynini], test_suite="pynini_test")
